;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Enemy Routines                                            ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; clearEnemyTable - Clear enemyTable
;
; Input:    None
; Output:   None
;------------------------------------------------
clearEnemyTable:
        ld      hl,enemyTable
        ld      de,enemyTable+1
        ld      bc,NUM_ENEMIES*7-1
        ld      (hl),0
        ldir
        ret

;------------------------------------------------
; clearAlienHoleTable - Clear alienHoleTable
;
; Input:    None
; Output:   None
;------------------------------------------------
clearAlienHoleTable:
        ld      hl,alienHoleTable
        ld      de,alienHoleTable+1
        ld      bc,NUM_ALIEN_HOLES*3-1
        ld      (hl),220
        ldir
        ret

;------------------------------------------------
; clearSecurityGuardTable - Clear securityGuardTable
;
; Input:    None
; Output:   None
;------------------------------------------------
clearSecurityGuardTable:
        ld      hl,securityGuardTable
        ld      de,securityGuardTable+1
        ld      bc,NUM_SECURITY_GUARDS*2-1
        ld      (hl),0
        ldir
        ret

;------------------------------------------------
; checkEnemyTriggers - Check to see if a tile is an enemy trigger
;
; Input:    A = Tile
;           C = 0 to remove trigger, >0 to leave trigger
; Output:   None
;------------------------------------------------
checkEnemyTriggers:
        sub     TRIGGER_TILE
        jp      z,newWorm
        dec     a
        jp      z,newFaceHugger
        dec     a
        jp      z,newNormalAlien
        dec     a
        jp      z,newStrongAlien
        ret

;------------------------------------------------
; newWorm - Try to create a new Worm
;
; Input:    HL => Trigger in level
; Output:   None
;------------------------------------------------
newWorm:
        ld      d,1
        jr      newEnemy

;------------------------------------------------
; newFaceHugger - Try to create a new Face Hugger
;
; Input:    HL => Trigger in level
; Output:   None
;------------------------------------------------
newFaceHugger:
        ld      d,2
        jr      newEnemy

;------------------------------------------------
; newNormalAlien - Try to create a new Normal Alien
;
; Input:    HL => Trigger in level
;           C = 0 to remove trigger, 1 to leave trigger
; Output:   None
;------------------------------------------------
newNormalAlien:
        ld      d,3
        jr      newEnemy

;------------------------------------------------
; newStrongAlien - Try to create a new Strong Alien
;
; Input:    HL => Trigger in level
;           C = 0 to remove trigger, 1 to leave trigger
; Output:   None
;------------------------------------------------
newStrongAlien:
        ld      d,4
        jr      newEnemy

;------------------------------------------------
; newSecurityGuard - Try to create a new Security Guard
;
; Input:    HL => Trigger in level
; Output:   None
;------------------------------------------------
newSecurityGuard:
        ld      d,5
        ld      c,1

;------------------------------------------------
; newEnemy - Try to create a new enemy
;
; Input:    HL => Trigger in level
;           C = 0 to remove trigger, 1 to leave trigger
;           D = Enemy type to create
; Output:   None
;------------------------------------------------
newEnemy:
        push    ix
        push    af
        push    hl
        ld      a,d
        ld      (_enemyType),a
        call    getEmptyEnemyEntry
        ld      d,$00
_enemyType      = $-1
        call    nc,enemyEntryFound
        pop     hl
        pop     af
        pop     ix
        ret

;------------------------------------------------
; enemyEntryFound - Create a new enemy
;
; Input:    IX => Entry to put enemy in
;           C = 0 to remove trigger, 1 to leave trigger
;           D = Type of enemy to create
;           HL => Trigger in level
; Output:   None
;------------------------------------------------
enemyEntryFound:
        dec     c
        jr      z,afterRemoveTrigger
        ld      a,(type)
        ld      (hl),a
afterRemoveTrigger:
        ld      (ix),d                          ; Set Type
        ld      (ix+1),0                        ; Set Direction
        call    getExactXYFromHL
        ld      (ix+2),h                        ; Set X
        ld      (ix+3),l                        ; Set Y
        ld      (ix+4),0                        ; Set Counter
        ld      l,(ix)
        dec     l
        ld      h,0
        TIMES8(hl)
        ld      de,enemyEnergyTable
        add     hl,de
        ld      a,(level)
        ld      e,a
        ld      d,0
        add     hl,de
        ld      a,(hl)
        ld      (ix+5),a                        ; Set Energy
        ld      b,16
        call    ionRandom
        ld      e,a
        ld      d,0
        ld      hl,enemySpeedTable
        add     hl,de
        push    hl
        ld      l,(ix)
        dec     l
        ld      h,0
        TIMES8(hl)
        add     hl,hl
        pop     de
        add     hl,de
        ld      a,(hl)
        ld      b,a
        add     a,a
        add     a,a
        or      b
        ld      (ix+6),a                        ; Set Speed
        ret

;------------------------------------------------
; getEmptyEnemyEntry - Get a pointer to an empty entry in enemyTable
;
; Input:    None
; Output:   IX => Entry (if one is found)
;           CA = 0 if successful, 1 if failed
;------------------------------------------------
getEmptyEnemyEntry:
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
        ld      de,7
findEmptyEnemyEntry:
        ld      a,(ix)
        or      a
        ret     z
        add     ix,de
        djnz    findEmptyEnemyEntry
        scf
        ret

;------------------------------------------------
; getNormalAlienSprite - Get pointer to Normal Alien sprite to show
;
; Input:    IX => Entry in enemyTable
; Output:   HL => Sprite
;------------------------------------------------
getNormalAlienSprite:
        ld      de,sprNormalAlienUp

;------------------------------------------------
; getNASASGSprite - Get pointer to Normal Alien/Strong Alien/Security Guard sprite to show
;
; Input:    IX => Entry in enemyTable
;           DE => First sprite
; Output:   HL => Sprite
;------------------------------------------------
getNASASGSprite:
        ld      l,(ix+1)
        ld      h,0
        TIMES8(hl)
        add     hl,de
        jr      afterGetEnemySprite

;------------------------------------------------
; getStrongAlienSprite - Get pointer to Strong Alien sprite to show
;
; Input:    IX => Entry in enemyTable
; Output:   HL => Sprite
;------------------------------------------------
getStrongAlienSprite:
        ld      de,sprStrongAlienUp
        jr      getNASASGSprite

;------------------------------------------------
; getSecurityGuardSprite - Get pointer to Security Guard sprite to show
;
; Input:    IX => Entry in enemyTable
; Output:   HL => Sprite
;------------------------------------------------
getSecurityGuardSprite:
        ld      de,sprSecurityGuardUp
        jr      getNASASGSprite

;------------------------------------------------
; getWormSprite - Get pointer to Worm sprite to show
;
; Input:    IX => Entry in enemyTable
; Output:   HL => Sprite
;------------------------------------------------
getWormSprite:
        ld      de,sprWormUp1

;------------------------------------------------
; getWFHSprite - Get pointer to Worm/Face Hugger sprite to show
;
; Input:    IX => Entry in enemyTable
;           DE => First sprite
; Output:   HL => Sprite
;------------------------------------------------
getWFHSprite:
        ld      l,(ix+1)
        ld      h,0
        TIMES8(hl)
        add     hl,de
        ex      de,hl
        ld      l,(ix+4)
        ld      h,0
        TIMES8(hl)
        add     hl,hl
        add     hl,hl
        add     hl,de
        jr      afterGetEnemySprite

;------------------------------------------------
; getFaceHuggerSprite - Get pointer to Face Hugger sprite to show
;
; Input:    IX => Entry in enemyTable
; Output:   HL => Sprite
;------------------------------------------------
getFaceHuggerSprite:
        ld      de,sprFaceHuggerUp1
        jr      getWFHSprite

;------------------------------------------------
; drawEnemies - Draw enemies
;
; Input:    None
; Output:   None
;------------------------------------------------
drawEnemies:
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
drawEnemiesLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endDrawEnemiesLoop
        dec     a
        jr      z,getWormSprite
        dec     a
        jr      z,getFaceHuggerSprite
        dec     a
        jr      z,getNormalAlienSprite
        dec     a
        jr      z,getStrongAlienSprite
        jr      getSecurityGuardSprite
afterGetEnemySprite:
        ex      de,hl
        ld      a,(ix+2)
        ld      hl,scrX
        sub     (hl)
        ld      b,a
        inc     hl
        ld      a,(ix+3)
        sub     (hl)
        ld      c,a
        ex      de,hl
        push    ix
        call    putClippedSprite
        pop     ix
endDrawEnemiesLoop:
        pop     bc
        ld      de,7
        add     ix,de
        djnz    drawEnemiesLoop
        ret

;------------------------------------------------
; moveEnemies - Move enemies
;
; Input:    None
; Output:   None
;------------------------------------------------
moveEnemies:
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
moveEnemiesLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endMoveEnemiesLoop
        ld      a,(ix+6)
        and     $03
        ld      b,a
        ld      a,(ix+6)
        and     $0C
        dec     b
        jr      nz,noMoveEnemy
        ld      b,a
        srl     b
        srl     b
        or      b
        ld      (ix+6),a
        ld      a,(x)
        sub     (ix+2)
        jr      c,moveEnemyLeft
        jr      nz,moveEnemyRight
checkEnemyY:
        ld      a,(y)
        sub     (ix+3)
        jr      c,moveEnemyUp
        jp      nz,moveEnemyDown
endMoveEnemiesLoop:
        pop     bc
        ld      de,7
        add     ix,de
        djnz    moveEnemiesLoop
        ret
noMoveEnemy:
        or      b
        ld      (ix+6),a
        jr      endMoveEnemiesLoop
moveEnemyLeft:
        ld      (ix+1),2
        ld      a,(ix+3)
        ld      (_enemyY),a
        inc     a
        ld      l,a
        ld      a,(ix+2)
        dec     a
        ld      (_enemyX),a
        inc     a
        call    getTileSave
        cp      SOLID_TILE
        jr      nc,checkEnemyY
        ld      a,l
        add     a,5
        ld      l,a
        ld      a,h
        call    getTileSave
        cp      SOLID_TILE
        jr      nc,checkEnemyY
finishMoveEnemyX:
        call    checkEnemyCollisions
        jr      nc,checkEnemyY
        ld      a,(_enemyX)
        ld      (ix+2),a
        call    incEnemyAnim
        jr      endMoveEnemiesLoop
moveEnemyRight:
        ld      (ix+1),3
        ld      a,(ix+3)
        ld      (_enemyY),a
        inc     a
        ld      l,a
        ld      a,(ix+2)
        inc     a
        ld      (_enemyX),a
        add     a,6
        call    getTileSave
        cp      SOLID_TILE
        jr      nc,checkEnemyY
        ld      a,l
        add     a,5
        ld      l,a
        ld      a,h
        call    getTileSave
        cp      SOLID_TILE
        jr      nc,checkEnemyY
        jr      finishMoveEnemyX
moveEnemyUp:
        ld      (ix+1),0
        ld      a,(ix+3)
        dec     a
        ld      (_enemyY),a
        inc     a
        ld      l,a
        ld      a,(ix+2)
        ld      (_enemyX),a
        inc     a
        call    getTileSave
        cp      SOLID_TILE
        jp      nc,endMoveEnemiesLoop
        ld      a,h
        add     a,5
        call    getTileSave
        cp      SOLID_TILE
        jp      nc,endMoveEnemiesLoop
finishMoveEnemyY:
        call    checkEnemyCollisions
        jp      nc,endMoveEnemiesLoop
        ld      a,(_enemyY)
        ld      (ix+3),a
        call    incEnemyAnim
        jp      endMoveEnemiesLoop
moveEnemyDown:
        ld      (ix+1),1
        ld      a,(ix+3)
        inc     a
        ld      (_enemyY),a
        add     a,6
        ld      l,a
        ld      a,(ix+2)
        ld      (_enemyX),a
        inc     a
        call    getTileSave
        cp      SOLID_TILE
        jp      nc,endMoveEnemiesLoop
        ld      a,h
        add     a,5
        call    getTileSave
        cp      SOLID_TILE
        jp      nc,endMoveEnemiesLoop
        jr      finishMoveEnemyY
incEnemyAnim:
        ld      a,(ix+4)
        inc     a
        and     $03
        ld      (ix+4),a
        ret

;------------------------------------------------
; checkEnemyCollisions - Check for collisions between enemies
;
; Input:    (_enemyX) = X Coord of enemy to check for
;           (_enemyY) = Y Coord of enemy to check for
; Output:   CA = Enemy not colliding with other enemies
;------------------------------------------------
checkEnemyCollisions:
        push    ix
        ld      ix,enemyTable
        ld      bc,NUM_ENEMIES*256+0
checkEnemyCollisionsLoop:
        ld      a,(ix)
        or      a
        jr      z,endCECLoop
        ld      a,$00
_enemyX         = $-1
        sub     (ix+2)
        jr      nc,cecXOK
        neg
cecXOK:
        cp      8
        jr      nc,endCECLoop
        ld      a,$00
_enemyY         = $-1
        sub     (ix+3)
        jr      nc,cecYOK
        neg
cecYOK:
        cp      8
        jr      nc,endCECLoop
        inc     c
endCECLoop:
        ld      de,7
        add     ix,de
        djnz    checkEnemyCollisionsLoop
        ld      a,c
        cp      2
        pop     ix
        ret

;------------------------------------------------
; enemyDead - Kill an enemy and initiate an explosion animation
;
; Input:    IX => Enemy entry in enemyTable
; Output:   IX => Enemy entry in enemyTable
;------------------------------------------------
enemyDead:
        ld      l,(ix)
        dec     l
        ld      h,0
        add     hl,hl
        ld      de,enemyScoreTable
        add     hl,de
        ld      e,(hl)
        inc     hl
        ld      d,(hl)
        call    addScore
        ld      (ix),0
        ld      h,(ix+2)
        ld      l,(ix+3)
        ld      bc,4*256+1
        jp      newAnim

;------------------------------------------------
; doSecurityGuardMissile - Update Security Guard shooting
;
; Input:    None
; Output:   None
;------------------------------------------------
doSecurityGuardMissile:
        ld      a,(securityGuardMissile)
        or      a
        jr      nz,moveSGMissile
        ld      a,(frame)
        and     $0F
        ret     nz
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
findSecurityGuard:
        ld      a,(ix)
        cp      5
        jr      nz,nextSecurityGuard
        ld      a,(x)
        sub     (ix+2)
        jr      nc,sgAbsX
        neg
sgAbsX:
        cp      5
        jr      c,securityGuardShoot
        ld      a,(y)
        sub     (ix+3)
        jr      nc,sgAbsY
        neg
sgAbsY:
        cp      5
        jr      c,securityGuardShoot
nextSecurityGuard:
        ld      de,7
        add     ix,de
        djnz    findSecurityGuard
        ret
moveSGMissile:
        ld      ix,securityGuardMissile
        ld      b,1
        jp      moveBulletsLoop
securityGuardShoot:
        push    ix
        pop     hl
        inc     hl
        ld      b,h
        ld      c,l
        inc     bc
        ld      ix,securityGuardMissile
        ld      a,5
        jp      makeNewBullet

;------------------------------------------------
; drawSecurityGuardMissile - Draw Security Guard missile
;
; Input:    None
; Output:   None
;------------------------------------------------
drawSecurityGuardMissile:
        ld      a,(securityGuardMissile)
        or      a
        ret     z
        ld      ix,securityGuardMissile
        ld      b,1
        jp      drawBulletsLoop

.end
