;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Level Loading Routines                                    ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; loadLevel - Load a level from current Levelset
;
; Input:    A = Level to load
; Output:   None
;------------------------------------------------
loadLevel:
        GETTEMP(hl)
        push    hl
        ld      d,h
        ld      e,l
        inc     de
        ld      bc,LEVEL_SIZE-1
        ld      (hl),0
        ldir
        ld      hl,(dataPtr)
        pop     de
        push    de
        ld      ix,gbuf
        ld      b,a
        call    huffExtr
        pop     hl
        push    hl
        ld      de,levelData
        ld      bc,LEVEL_HEADER_LENGTH
        ldir
        pop     de
        push    de
        ld      bc,LEVEL_SIZE
        ldir
        ld      hl,height
        dec     (hl)
        inc     hl
        dec     (hl)
        pop     hl
; Search for Alien Holes & Security Guard triggers
        di
        push    iy
        ld      bc,LEVEL_SIZE-LEVEL_HEADER_LENGTH
        ld      ix,alienHoleTable
        ld      iy,securityGuardTable
searchForHolesAndTriggers:
        push    bc
        push    hl
        ld      a,(hl)
        cp      OPEN_ALIEN_HOLE_TILE
        jr      nz,notAlienHole
        ld      (ix),0
        call    getExactXYFromHL
        ld      (ix+1),a
        ld      (ix+2),l
        inc     ix
        inc     ix
        inc     ix
        jr      endSearchForHolesAndTriggers
notAlienHole:
        cp      SECURITY_GUARD_TILE
        jr      nz,endSearchForHolesAndTriggers
        call    getExactXYFromHL
        ld      (iy),a
        ld      (iy+1),l
        inc     iy
        inc     iy
endSearchForHolesAndTriggers:
        pop     hl
        inc     hl
        pop     bc
        dec     bc
        ld      a,c
        or      b
        jr      nz,searchForHolesAndTriggers
        pop     iy
        ei
; Show level info
        bcall(_grbufclr)
        ld      hl,strLevelInfo
#ifdef TI86
        ld      de,0*256+40
        call    vPuts                           ; "ALIEN BREED IV:"
        ld      de,7*256+40
        call    vPuts                           ; "FINAL ASSAULT"
        ld      de,20*256+21
        call    vPuts                           ; "LEVELSET:"
        ld      de,27*256+21
        call    vPuts                           ; "HI-SCORE:"
        ld      de,37*256+21
        call    vPuts                           ; "MISSION:"
        ld      de,44*256+21
        call    vPuts                           ; "PASSCODE:"
        ld      de,57*256+41
        call    vPuts                           ; "PRESS ANY KEY
        ld      hl,levelsetName
        ld      de,20*256+96
        call    vPuts                           ; Levelset name
        ld      hl,(hiScore)
        ld      de,27*256+88
        call    setShowHL                       ; Hi-Score
        ld      hl,(level)
        ld      h,0
        inc     l
        ld      de,37*256+88
        call    setShowHL                       ; Mission number
        ld      a,(level)
        or      a
        jr      z,noPasscodeToShow
        dec     a
        add     a,a
        ld      c,a
        add     a,a
        add     a,c
        ld      e,a
        ld      d,0
        ld      hl,(passcodePtr)
        add     hl,de
        ld      de,44*256+88
        call    vPuts                           ; Passcode
noPasscodeToShow:
        ld      hl,gbuf+4
        ld      bc,14*256+8
        call    invertBox
        ld      hl,gbuf+(57*16)+4
        ld      bc,7*256+8
        call    invertBox
#else
        ld      de,0*256+23
        call    vPuts                           ; "ALIEN BREED IV:"
        ld      de,7*256+24
        call    vPuts                           ; "FINAL ASSAULT"
        ld      de,20*256+5
        call    vPuts                           ; "LEVELSET:"
        ld      de,27*256+5
        call    vPuts                           ; "HI-SCORE:"
        ld      de,37*256+5
        call    vPuts                           ; "MISSION:"
        ld      de,44*256+5
        call    vPuts                           ; "PASSCODE:"
        ld      de,57*256+25
        call    vPuts                           ; "PRESS ANY KEY
        ld      hl,levelsetName
        ld      de,20*256+80
        call    vPuts                           ; Levelset name
        ld      hl,(hiScore)
        ld      de,27*256+72
        call    setShowHL                       ; Hi-Score
        ld      hl,(level)
        ld      h,0
        inc     l
        ld      de,37*256+72
        call    setShowHL                       ; Mission number
        ld      a,(level)
        or      a
        jr      z,noPasscodeToShow
        dec     a
        add     a,a
        ld      c,a
        add     a,a
        add     a,c
        ld      e,a
        ld      d,0
        ld      hl,(passcodePtr)
        add     hl,de
        ld      de,44*256+72
        call    vPuts                           ; Passcode
noPasscodeToShow:
        ld      hl,gbuf+2
        ld      bc,14*256+8
        call    invertBox
        ld      hl,gbuf+(57*12)+2
        ld      bc,7*256+8
        call    invertBox
#endif
        call    ionFastCopy
        call    waitKeyA
; Fix tiles according to Level type
        ld      a,(type)
        ld      hl,tileBlank
        or      a
        jr      z,doFixTiles
        ld      hl,tileHatchery
doFixTiles:
        ld      (_fixTiles),hl
        ld      ix,tileFixTable
        ld      a,TILES_TO_FIX
fixTilesLoop:
        ld      e,(ix)
        ld      d,(ix+1)
        ld      b,(ix+2)
fixTilesInnerLoop:
        push    bc
        ld      hl,$0000
_fixTiles       = $-2
        ld      bc,8
        ldir
        pop     bc
        djnz    fixTilesInnerLoop
        ld      de,4
        add     ix,de
        dec     a
        jr      nz,fixTilesLoop
drawLevel:
        bcall(_grbufclr)
        call    getScrX
        call    getScrY
        xor     a
        ld      (_dlY),a
dlNextY:
        xor     a
        ld      (_dlX),a
        ld      a,(_dlY)
        ld      hl,scrY
        add     a,(hl)
        ld      hl,0
        or      a
        jr      z,dlSkip
        ld      hl,(width)
        inc     l
        ld      h,a
        bcall(_htimesl)
dlSkip:
        GETTEMP(de)
        add     hl,de
        ld      a,(scrX)
        ld      e,a
        ld      d,0
        add     hl,de
dlNextX:
        push    hl
        ld      c,0
        ld      a,(hl)
        call    checkEnemyTriggers
        ld      l,(hl)
        ld      h,0
        TIMES8(hl)
        ld      de,startTiles
        add     hl,de
        ex      de,hl
        ld      l,$00
_dlY            = $-1
        ld      a,$00
_dlX            = $-1
        call    putAlignedSprite
        pop     hl
        inc     hl
        ld      a,(_dlX)
        inc     a
        ld      (_dlX),a
        cp      12
        jr      nz,dlNextX
        ld      a,(_dlY)
        inc     a
        ld      (_dlY),a
        cp      8
        jr      nz,dlNextY
        ld      a,(scrX)
        TIMES8(a)
        ld      (scrX),a
        ld      a,(scrY)
        TIMES8(a)
        ld      (scrY),a
        ld      a,(x)
        TIMES8(a)
        ld      (x),a
        ld      a,(y)
        TIMES8(a)
        ld      (y),a
        ld      a,(scrX)
        or      a
        jr      z,dlNoScrollRight
        call    divAby8
        add     a,12
        ld      hl,(width)
        inc     l
        cp      l
        jr      z,dlNoScrollRight
        call    scrollRight
        call    scrollRight
        call    scrollRight
        call    scrollRight
dlNoScrollRight:
        ld      a,(scrY)
        or      a
        ret     z
        call    divAby8
        add     a,8
        ld      hl,(height)
        inc     l
        cp      l
        ret     z
        call    scrollDown
        call    scrollDown
        call    scrollDown
        jp      scrollDown
getScrX:
        ld      a,(x)
        sub     5
        jr      c,startLeft
        add     a,11
        ld      hl,(width)
        inc     l
        cp      l
        jr      nc,startRight
        sub     11
saveScrX:
        ld      (scrX),a
        ret
getScrY:
        ld      a,(y)
        sub     3
        jr      c,startTop
        add     a,7
        ld      hl,(height)
        inc     l
        cp      l
        jr      nc,startBottom
        sub     7
saveScrY:
        ld      (scrY),a
        ret
startLeft:
        xor     a
        jr      saveScrX
startRight:
        ld      a,(width)
        sub     11
        jr      saveScrX
startTop:
        xor     a
        jr      saveScrY
startBottom:
        ld      a,(height)
        sub     7
        jr      saveScrY

.end
