;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Main Loop                                                 ;
;                                                           ;
;-----------------------------------------------------------;

mainLoop:
;        BREAK()
; Reset timer, increment frame counter
        xor     a
        ld      (timer),a
        ld      hl,frame
        inc     (hl)
        ld      hl,dead
        ld      a,(hl)
        or      a
        jr      z,playerAlive
        dec     (hl)
        jr      nz,afterEndDeathSpin
        ld      a,80
        ld      (invincible),a
afterEndDeathSpin:
        ld      a,(hl)
        and     $03
        jr      nz,playerAlive
        ld      hl,(playerDir)
        ld      h,0
        ld      de,deathSpinTable
        add     hl,de
        ld      a,(hl)
        ld      (playerDir),a
playerAlive:
; See if level is finished
        ld      a,(objectives)
        cp      2
        jr      nc,levelNotFinished
        ld      a,(x)
        inc     a
        inc     a
        inc     a
        call    divAby8
        ld      hl,finishX
        cp      (hl)
        jr      nz,levelNotFinished
        ld      a,(y)
        inc     a
        inc     a
        inc     a
        call    divAby8
        inc     hl
        cp      (hl)
        jp      z,levelFinished
levelNotFinished:
; Update Alien Holes and Security Guard triggers
        ld      hl,alienHoleTable
        ld      b,NUM_ALIEN_HOLES
updateAlienHoles:
        push    bc
        push    hl
        ld      a,(hl)
        cp      225
        jr      c,maybeActiveAlienHole
        inc     (hl)
        jr      nz,endUpdateAlienHoles
        inc     hl
        ld      a,(hl)
        inc     hl
        ld      l,(hl)
        call    getTile
        ld      a,OPEN_ALIEN_HOLE_TILE
        call    putTile
        jr      endUpdateAlienHoles
maybeActiveAlienHole:
        cp      220
        jr      z,endUpdateAlienHoles
        inc     (hl)
        ld      a,(hl)
        sub     180
        jr      nz,endUpdateAlienHoles
        ld      (hl),a
        inc     hl
        ld      a,(hl)
        inc     hl
        ld      l,(hl)
        call    checkOnScreen
        jr      nc,endUpdateAlienHoles
        ld      (_enemyX),a
        ld      a,l
        ld      (_enemyY),a
        ld      a,h
        call    getTile
        call    checkEnemyCollisions
        cp      1
        jr      nc,endUpdateAlienHoles
        ld      c,1
        ld      a,(alienType)
        ld      d,a
        call    newEnemy
endUpdateAlienHoles:
        pop     hl
        inc     hl
        inc     hl
        inc     hl
        pop     bc
        djnz    updateAlienHoles
; Update Security Guard triggers
        ld      hl,securityGuardCnt
        inc     (hl)
        jr      nz,afterUpdateSG
        ld      hl,securityGuardTable
        ld      b,NUM_SECURITY_GUARDS
updateSGLoop:
        push    hl
        push    bc
        ld      a,(hl)
        or      a
        jr      z,endUpdateSGLoop
        ld      a,(hl)
        inc     hl
        ld      l,(hl)
        call    checkOnScreen
        jr      nc,endUpdateSGLoop
        ld      (_enemyX),a
        ld      a,l
        ld      (_enemyY),a
        ld      a,h
        call    getTile
        call    checkEnemyCollisions
        cp      1
        call    c,newSecurityGuard
endUpdateSGLoop:
        pop     bc
        pop     hl
        inc     hl
        inc     hl
        djnz    updateSGLoop
afterUpdateSG:
; Update invincibility
        ld      hl,invincible
        ld      a,(hl)
        or      a
        jr      z,afterDecInvincible
        dec     (hl)
afterDecInvincible:
; Check to see if the objectives counter is on countdown mode
        ld      hl,objectives
        ld      a,(hl)
        cp      50
        jr      z,setObjectivesTo1
        jr      c,afterCheckDecObjectives
        dec     (hl)
        jr      afterCheckDecObjectives
setObjectivesTo1:
        ld      (hl),1
afterCheckDecObjectives:
; Check to see if we should decrement the time left
        ld      a,(hl)
        dec     a
        jr      nz,afterDecTime
        ld      a,(frame)
        and     $1F
        jr      nz,afterDecTime
        ld      hl,time
        dec     (hl)
        ld      a,(hl)
        inc     a
        jp      z,gameOver
afterDecTime:
        ld      hl,(playerCenter)
        ld      a,h
        call    getTile
        cp      VFIRE_DOOR_TILE
        jr      z,onFireDoor
        cp      HFIRE_DOOR_TILE
        jr      z,onFireDoor
        ld      b,a
        ld      a,(fireDoor+1)
        or      a
        ld      a,b
        jr      z,afterFireDoor
        ld      hl,(fireDoor)
        ld      a,(hl)
        add     a,CLOSE_FIRE_DOOR
        call    putTile
        xor     a
        ld      (fireDoor+1),a
        call    lowerObjectives
        jr      afterTileChecks
onFireDoor:
        ld      (fireDoor),hl
        jr      afterTileChecks
afterFireDoor:
        cp      AIR_DUCT_TILE
        jr      nz,afterAirDuct
        call    checkCanGetHurt
        jr      nz,afterAirDuct
        ld      a,-1
        ld      (health),a
afterAirDuct:
afterTileChecks:
        call    moveEnemies
        call    doSecurityGuardMissile
        ld      a,(frame)
        bit     0,a
        call    z,updateAnims
        call    moveBullets
        ld      a,(queen)
        or      a
        call    nz,moveQueen

        ld      a,(hurt)
        or      a
        jr      nz,afterPlayerMovement
        ld      a,(dead)
        or      a
        jr      nz,afterKeyChecks

; Check key presses
        ld      a,KG_ARROW
        call    directIn
        push    af
        bit     DI_UP,a
        call    z,playerUp
        pop     af
        push    af
        bit     DI_DOWN,a
        call    z,playerDown
        pop     af
        push    af
        bit     DI_LEFT,a
        call    z,playerLeft
        pop     af
        bit     DI_RIGHT,a
        call    z,playerRight

afterPlayerMovement:
        xor     a
        ld      (hurt),a

        ld      a,KG_1
        call    directIn
        push    af
        bit     DI_ALPHA,a
        call    z,doIntexConsole
        pop     af
#ifdef TI86
        bit     DI_GRAPH,a
#else
        bit     DI_MATH,a
#endif
        call    z,teacherKey

        ld      a,KG_TOP
        call    directIn
        push    af
        bit     DI_F1,a
        call    z,selectWeapon1
        pop     af
        push    af
        bit     DI_F2,a
        call    z,selectWeapon2
        pop     af
        push    af
        bit     DI_F3,a
        call    z,selectWeapon3
        pop     af
        push    af
        bit     DI_F4,a
        call    z,selectWeapon4
        pop     af
        push    af
        bit     DI_F5,a
        call    z,selectWeapon5
        pop     af
        push    af
        bit     DI_2ND,a
        call    z,shoot
        pop     af
        push    af
#ifdef TI86
        bit     DI_MORE,a
#else
        bit     DI_DEL,a
#endif
        call    z,viewRadarScanner
        pop     af
#ifdef TI86
        bit     DI_EXIT,a
#else
        bit     DI_MODE,a
#endif
        call    z,pause

afterKeyChecks:
; Save center position of player
        ld      a,(x)
        inc     a
        inc     a
        inc     a
        ld      hl,(y)
        inc     l
        inc     l
        inc     l
        ld      h,a
        ld      (playerCenter),hl
; Check for pickups
        ld      hl,(x)
        ld      a,l
        ld      l,h
        call    checkPickups
        add     a,7
        call    checkPickups
        ld      b,a
        ld      a,l
        add     a,7
        ld      l,a
        ld      a,b
        call    checkPickups
        sub     7
        call    checkPickups

        call    checkBulletEnemyCollisions
        call    checkEnemyPlayerCollisions
        call    checkPlayerSGMissileCollisions
        ld      a,(queen)
        or      a
        jr      z,noCheckQueenCollisions
        call    checkBulletQueenCollisions
        call    checkPlayerQueenCollisions
noCheckQueenCollisions:
; Check tile player is standing on
        ld      a,(y)
        inc     a
        inc     a
        inc     a
        ld      l,a
        ld      a,(x)
        inc     a
        inc     a
        inc     a
        call    getTile
        cp      QUEEN1_TILE
        jr      c,afterCheckTile
        cp      QUEEN3_TILE+1
        jp      c,iniQueen
afterCheckTile:
; Save background, draw sprites, show gbuf and restore background without sprites
        call    saveVideo
        ld      a,(queen)
        or      a
        call    nz,drawQueen
        call    drawAnims
        call    drawBullets
        call    drawSecurityGuardMissile
        call    drawEnemies
        call    drawPlayer
        ld      a,(objectives)
        dec     a
        jr      nz,afterShowTimeLeft
        ld      de,0
        ld      hl,(time)
        ld      h,0
        call    setShowHL
afterShowTimeLeft:
        call    videoCenterCopy
        ei
        call    restoreVideo

        ld      hl,timer
        jr      noHalt
mainLoopWait:
        halt
noHalt:
        ld      a,(hl)
        neg
#ifdef TI86
        cp      6
#else
        cp      4
#endif

        ld      a,(health)
        cp      1
        jp      p,mainLoop
        ld      a,48
        ld      (dead),a
        ld      a,100
        ld      (health),a
        ld      hl,lives
        dec     (hl)
        bit     7,(hl)
        jp      nz,gameOver
        jp      mainLoop

.end
