;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Player Routines                                           ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; playerUp - Try to move player up
;
; Input:    None
; Output:   None
;------------------------------------------------
playerUp:
; Set direction
        xor     a
        ld      (playerDir),a
; Check if a solid tile is in the way
        ld      a,(x)
        inc     a
        ld      hl,(y)
        call    checkTile
        ret     c
        ld      a,(x)
        add     a,6
        ld      hl,(y)
        call    checkTile
        ret     c
; Check if at top of level
        ld      hl,y
        ld      a,(hl)
        or      a
        ret     z
; Move up
        dec     (hl)
; Check if should scroll screen
        ld      a,(scrY)
        or      a
        ret     z
        ld      b,a
        ld      a,(hl)
        sub     b
        cp      24
        ret     nc
        jp      scrollDown

;------------------------------------------------
; playerDown - Try to move player down
;
; Input:    None
; Output:   None
;------------------------------------------------
playerDown:
; Set direction
        ld      a,1
        ld      (playerDir),a
; Check if a solid tile is in the way
        ld      a,(y)
        add     a,7
        ld      l,a
        ld      a,(x)
        inc     a
        call    checkTile
        ret     c
        ld      a,(y)
        add     a,7
        ld      l,a
        ld      a,(x)
        add     a,6
        call    checkTile
        ret     c
; Check if at bottom of level
        ld      hl,y
        ld      a,(height)
        TIMES8(a)
        cp      (hl)
        ret     z
; Move down
        inc     (hl)
; Check if should scroll screen
        ld      a,(scrY)
        call    divAby8
        add     a,8
        ld      hl,(height)
        inc     l
        cp      l
        ret     z
        ld      a,(y)
        ld      hl,scrY
        sub     (hl)
        cp      33
        ret     c
        jp      scrollUp

;------------------------------------------------
; playerLeft - Try to move player left
;
; Input:    None
; Output:   None
;------------------------------------------------
playerLeft:
; Set direction
        ld      a,2
        ld      (playerDir),a
; Check if a solid tile is in the way
        ld      a,(x)
        ld      hl,(y)
        inc     l
        call    checkTile
        ret     c
        ld      a,(y)
        add     a,6
        ld      l,a
        ld      a,(x)
        call    checkTile
        ret     c
; Check if at far left of level
        ld      hl,x
        ld      a,(hl)
        or      a
        ret     z
; Move left
        dec     (hl)
; Check if should scroll screen
        ld      a,(scrX)
        or      a
        ret     z
        ld      b,a
        ld      a,(hl)
        sub     b
        cp      40
        ret     nc
        jp      scrollRight

;------------------------------------------------
; playerRight - Try to move player right
;
; Input:    None
; Output:   None
;------------------------------------------------
playerRight:
; Set direction
        ld      a,3
        ld      (playerDir),a
; Check if a solid tile is in the way
        ld      a,(x)
        add     a,7
        ld      hl,(y)
        inc     l
        call    checkTile
        ret     c
        ld      a,(y)
        add     a,6
        ld      l,a
        ld      a,(x)
        add     a,7
        call    checkTile
        ret     c
; Check if at far right of level
        ld      hl,x
        ld      a,(width)
        TIMES8(a)
        cp      (hl)
        ret     z
; Move right
        inc     (hl)
; Check if should scroll screen
        ld      a,(scrX)
        call    divAby8
        add     a,12
        ld      hl,(width)
        inc     l
        cp      l
        ret     z
        ld      a,(x)
        ld      hl,scrX
        sub     (hl)
        cp      49
        ret     c
        jp      scrollLeft

;------------------------------------------------
; drawPlayer - Draw player on gbuf
;
; Input:    None
; Output:   None
;------------------------------------------------
drawPlayer:
        ld      a,(invincible)
        bit     0,a
        ret     nz
        ld      a,(x)
        ld      hl,scrX
        sub     (hl)
        ld      b,a
        ld      a,(y)
        ld      hl,scrY
        sub     (hl)
        ld      c,a
        ld      a,(playerDir)
        TIMES8(a)
        add     a,a
        ld      e,a
        ld      d,0
        ld      hl,sprPlayer
        add     hl,de
        ld      a,(hurt)
        ld      de,64
        or      a
        jr      z,doDrawPlayer
        add     hl,de
doDrawPlayer:
        jp      putClippedMaskedSprite

;------------------------------------------------
; checkPickups - Check a specified point for any pickup items and adjust player vars accordingly
;
; Input:    A = X
;           L = Y
; Output:   A = X
;           L = Y
;------------------------------------------------
checkPickups:
        push    af
        push    hl
        call    getTile
        ld      b,h
        ld      c,l                             ; BC => Tile
        sub     PICKUP_TILE                     ; Key Pass
        jr      z,getKeyPass
        dec     a                               ; White Key Pass
        jr      z,getWhiteKeyPass
        dec     a                               ; Checkered Key Pass
        jr      z,getCheckeredKeyPass
        dec     a                               ; Black Key Pass
        jr      z,getBlackKeyPass
        dec     a                               ; Ammo Clip
        jr      z,getAmmoClip
        dec     a                               ; 10 Credits
        jr      z,get10Credits
        dec     a                               ; 100 Credits
        jr      z,get100Credits
        dec     a                               ; First Aid
        jr      z,getFirstAid
        dec     a                               ; Invincibility
        jr      z,getInvincibility
        dec     a                               ; Extra Life
        jr      z,getExtraLife
        pop     hl
        pop     af
        ret
getKeyPass:
        ld      hl,keyPasses
        inc     (hl)
        jr      getPickupScore
getWhiteKeyPass:
        ld      hl,keyTable
        ld      (hl),1
        jr      getPickupScore
getCheckeredKeyPass:
        ld      hl,keyTable+1
        ld      (hl),1
        jr      getPickupScore
getBlackKeyPass:
        ld      hl,keyTable+2
        ld      (hl),1
        jr      getPickupScore
getAmmoClip:
        ld      hl,(ammo)
        ld      de,20
        add     hl,de
        ld      (ammo),hl
        jr      getPickupScore
get10Credits:
        ld      hl,(credits)
        ld      de,10
        add     hl,de
        ld      (credits),hl
        jr      getPickupScore
get100Credits:
        ld      hl,(credits)
        ld      de,100
        add     hl,de
        ld      (credits),hl
        jr      getPickupScore
getFirstAid:
        ld      hl,health
        ld      a,20
        add     a,(hl)
        ld      (hl),a
        cp      101
        jr      c,getPickupScore
        ld      (hl),100
        jr      getPickupScore
getInvincibility:
        ld      a,200
        ld      (invincible),a
        jr      getPickupScore
getExtraLife:
        ld      hl,lives
        inc     (hl)
getPickupScore:
        ld      de,20
        call    addScore
        ld      h,b
        ld      l,c
        ld      a,(type)
        call    putTile
        pop     hl
        pop     af
        ret

;------------------------------------------------
; checkTile - Check a tile to see if it's a wall, trigger or whatever
;
; Input:    A = X
;           L = Y
; Output:   CA = 1 if can't walk through
;------------------------------------------------
checkTile:
        call    getTile
        cp      BLACK_DOOR
        jr      nc,openBlackDoor
        cp      CHECKERED_DOOR
        jr      nc,openCheckeredDoor
        cp      WHITE_DOOR
        jr      nc,openWhiteDoor
        cp      SECURITY_DOOR
        jr      nc,openSecurityDoor
        cp      NORMAL_DOOR
        jr      nc,openNormalDoor
        cp      AUTO_DESTRUCT_TILE
        jr      z,triggerAutoDestruct
        cp      UNOPEN_ALIEN_HOLE_TILE
        jp      z,activateAlienHole
        cp      BOMB_TILE
        jr      z,triggerBomb
        cp      SOLID_TILE
        jr      c,maybeEnergyShield
checkTileNoMove:
        scf
        ret
openBlackDoor:
        ld      a,(keyTable+2)
        ld      b,OPEN_BLACK_DOOR
        jr      openColoredDoor
openCheckeredDoor:
        ld      a,(keyTable+1)
        ld      b,OPEN_CHECKERED_DOOR
        jr      openColoredDoor
openWhiteDoor:
        ld      a,(keyTable)
        ld      b,OPEN_WHITE_DOOR
openColoredDoor:
        or      a
        jr      z,checkTileNoMove
        ld      a,(hl)
        sub     b
        push    af
        call    putTile
        call    getExactXYFromHL
        pop     bc
        ld      a,b
        sub     ANIM_COLORED_DOOR
        ld      c,a
        ld      b,2
        call    newAnim
        jr      checkTileMove
openSecurityDoor:
        ld      a,(keyPasses)
        or      a
        jr      z,checkTileNoMove
        dec     a
        ld      (keyPasses),a
        ld      a,(hl)
        sub     OPEN_SECURITY_DOOR
        push    af
        call    putTile
        call    getExactXYFromHL
        pop     bc
        ld      a,b
        sub     ANIM_SECURITY_DOOR
        ld      c,a
        ld      b,2
        call    newAnim
        jr      checkTileMove
openNormalDoor:
        sub     OPEN_NORMAL_DOOR
        push    af
        call    putTile
        call    getExactXYFromHL
        pop     bc
        ld      a,b
        sub     ANIM_NORMAL_DOOR
        ld      c,a
        ld      b,2
        call    newAnim
        jr      checkTileMove
maybeEnergyShield:
        cp      ENERGY_SHIELD
        jr      c,checkTileMove
        sub     ENERGY_SHIELD
        ld      hl,playerDir
        cp      (hl)
        jr      nz,checkTileNoMove
checkTileMove:
        or      a
        ret
triggerAutoDestruct:
        xor     a
        jr      doTriggerStuff
triggerBomb:
        ld      a,TRIGGERED_BOMB_TILE
doTriggerStuff:
        call    putTile
        call    lowerObjectives
        jr      checkTileNoMove
activateAlienHole:
        ld      a,(type)
        ld      (hl),a
        ld      de,alienHoleTable
        ld      b,NUM_ALIEN_HOLES
findAlienHoleEntry:
        ld      a,(de)
        cp      220
        jr      z,alienHoleEntryFound
        inc     de
        inc     de
        inc     de
        djnz    findAlienHoleEntry
alienHoleEntryFound:
        ld      a,225
        ld      (de),a
        push    de
        call    getExactXYFromHL
        pop     de
        inc     de
        ld      (de),a
        inc     de
        ld      a,l
        ld      (de),a
        jr      checkTileMove

;------------------------------------------------
; selectWeaponX - Select a weapon
;
; Input:    None
; Output:   None
;------------------------------------------------
selectWeapon1:
        ld      b,1
        ld      a,b
selectWeapon:
        or      a
        ret     z
        ld      a,b
        ld      (weapon),a
        ret
selectWeapon2:
        ld      a,(weaponTable+1)
        ld      b,2
        jr      selectWeapon
selectWeapon3:
        ld      a,(weaponTable+2)
        ld      b,3
        jr      selectWeapon
selectWeapon4:
        ld      a,(weaponTable+3)
        ld      b,4
        jr      selectWeapon
selectWeapon5:
        ld      a,(weaponTable+4)
        ld      b,5
        jr      selectWeapon

;------------------------------------------------
; viewRadarScanner - View radar scanner (if own one)
;
; Input:    None
; Output:   None
;------------------------------------------------
viewRadarScanner:
        ld      a,(radarScanner)
        or      a
        ret     z
        call    saveVideo
        call    showMap
        jp      restoreVideo

;------------------------------------------------
; addScore - Add to the players score
;
; Input:    DE = How much to add
; Output:   HL = New score
;           DE = How much was added
;------------------------------------------------
addScore:
        ld      hl,(score)
        add     hl,de
        ld      (score),hl
        ret

;------------------------------------------------
; checkCanGetHurt - Check to see if player can get hurt
;
; Input:    None
; Output:   Z = Player can get hurt
;------------------------------------------------
checkCanGetHurt:
        ld      a,(dead)
        or      a
        ret     nz
        ld      a,(invincible)
        or      a
        ret

.end
