;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Queen Alien Routines                                      ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; iniQueen - Initialise new Queen Alien
;
; Input:    A = Trigger value
;           HL => Trigger in level data
; Output:   None
;------------------------------------------------
iniQueen:
        push    hl
        sub     QUEEN1_TILE
        add     a,a
        add     a,a
        ld      e,a
        ld      d,0
        ld      hl,queenDataTable
        add     hl,de
        ld      de,queenSprPtr
        ld      bc,4
        ldir
        pop     hl
        ld      a,(type)
        ld      (hl),a
        ld      hl,(blockPutY)
        ld      a,(blockPutX)
        ld      h,0
        TIMES8(hl)
        TIMES8(a)
        call    getTile
        ld      a,(blockPutType)
        call    putTile
        xor     a
        ld      (queenCnt),a
        inc     a
        ld      (queen),a
        ld      (queenSprCnt),a
        ld      (queenDir),a
        ld      a,(x)
        ld      (qX),a
        ld      a,(y)
        call    divAby8
        sub     9
        TIMES8(a)
        ld      (qY),a
        jp      afterCheckTile

;------------------------------------------------
; queenDead - Kill the Queen and return to normal play
;
; Input:    None
; Output:   None
;------------------------------------------------
queenDead:
        pop     hl
        pop     iy
        ei
        ld      de,1000
        call    addScore
        xor     a
        ld      (queen),a
        ld      hl,(blockTakeY)
        ld      a,(blockTakeX)
        ld      h,0
        TIMES8(hl)
        TIMES8(a)
        call    getTile
        ld      a,(blockTakeType)
        call    putTile
        jp      lowerObjectives

;------------------------------------------------
; drawQueen - Draw Queen Alien and its Health Bar
;
; Input:    None
; Output:   None
;------------------------------------------------
drawQueen:
        ld      a,(queenSprCnt)
        srl     a
        srl     a
        and     $03
        ld      l,a
        ld      h,0
        TIMES8(hl)
        add     hl,hl
        add     hl,hl
        ld      d,h
        ld      e,l
        add     hl,hl
        add     hl,de
        ld      de,(queenSprPtr)
        add     hl,de
        ex      de,hl
        ld      a,(qX)
        ld      hl,scrX
        sub     (hl)
        ld      (_qX),a
        ld      a,(qY)
        inc     hl
        sub     (hl)
        ld      (_qY),a
        ex      de,hl
        ld      b,6
        ld      ix,queenDrawTable
drawQueenLoop:
        push    bc
        push    ix
        push    hl
        ld      a,$00
_qY             = $-1
        add     a,(ix)
        ld      c,a
        ld      a,$00
_qX             = $-1
        add     a,(ix+1)
        ld      b,a
        call    putClippedMaskedSprite
        pop     hl
        ld      de,16
        add     hl,de
        pop     ix
        inc     ix
        inc     ix
        pop     bc
        djnz    drawQueenLoop
; Draw Health Bar
#ifdef TI86
        ld      hl,gbuf+(62*16)
        ld      de,gbuf+(62*16)+1
        ld      bc,2*16-1
#else
        ld      hl,gbuf+(62*12)
        ld      de,gbuf+(62*12)+1
        ld      bc,2*12-1
#endif
        ld      (hl),0
        ldir
        ld      hl,(queenHealth)
        ld      a,29
        bcall(_divhlbya)
        ld      a,l
queenHealthLoop:
        push    af
        ld      e,63
        call    ionGetPixel
        or      (hl)
        ld      (hl),a
        pop     af
        dec     a
        cp      255
        jr      nz,queenHealthLoop
        ret

;------------------------------------------------
; moveQueen - Move Queen Alien
;
; Input:    None
; Output:   None
;------------------------------------------------
moveQueen:
        ld      hl,queenSprCnt
        inc     (hl)
        ld      a,(queenDir)
        or      a
        ld      hl,qY
        jr      z,moveQueenUp
moveQueenDown:
        inc     (hl)
        ld      hl,queenCnt
        inc     (hl)
        ld      a,(hl)
        sub     8*7
        jr      nz,moveQueenX
        ld      (queenDir),a
        jr      moveQueenX
moveQueenUp:
        dec     (hl)
        ld      hl,queenCnt
        dec     (hl)
        jr      nz,moveQueenX
        inc     a
        ld      (queenDir),a
moveQueenX:
        ld      a,(queenSprCnt)
        bit     0,a
        ret     nz
        ld      a,(qX)
        add     a,7
        ld      b,a
        ld      a,(x)
        inc     a
        inc     a
        inc     a
        sub     b
        ld      b,0
        jr      nc,qAbsX
        neg
        inc     b
qAbsX:
        cp      5
        ret     c
        ld      hl,qX
        ld      a,b
        or      a
        jr      nz,moveQueenLeft
moveQueenRight:
        inc     (hl)
        ret
moveQueenLeft:
        dec     (hl)
        ret

.end
